package net.javase.hksup.starter.model;

import com.alibaba.fastjson2.annotation.JSONField;
import lombok.*;

import java.io.Serializable;
import java.util.List;

/**
 * UserInfo
 *
 * @author Frank
 */
@Getter
@Setter
@Builder
public class UserInfo implements Serializable {

    /**
     * 工号 (人员ID)
     */
    private String employeeNo;

    /**
     * 姓名
     */
    private String name;

    /**
     * 人员类型
     * <p>
     * [normal#普通人（主人）,visitor#来宾（访客）,blackList#黑名单人,maintenance#维护人员（包括保洁、维修人员 等),patient#病患]
     * <p>
     * 维护人员需要在任意时间都能进入房间
     */
    @Builder.Default
    private String userType = "normal";

    @Builder.Default
    @JSONField(name = "Valid")
    private Valid valid = new Valid();

    @Builder.Default
    private String doorRight = "1";

    @JSONField(name = "RightPlan")
    private List<RightPlan> rightPlan;

    /**
     * 人脸图片对应的人员性别
     * <p>
     * [male#男,female#女,unknown#未知]
     */
    @Builder.Default
    private String gender = "male";

    /**
     * 是否关门延迟
     */
    @Builder.Default
    private Boolean localUIRight = false;


    @Getter
    @Setter
    @NoArgsConstructor
    @AllArgsConstructor
    public static class Valid {
        /**
         * ture有 有效期, false长期有效
         */
        private Boolean enable = true;

        /**
         * 有效期起始时间
         */
        private String beginTime = "2023-01-01T00:00:00";

        /**
         * 有效期结束时间
         */
        private String endTime = "2037-12-31T23:59:59";

        /**
         * 时间类型[local#设备本地时间,UTC#UTC时间]
         */
        private String timeType = "local";

    }


    @Getter
    @Setter
    @AllArgsConstructor
    public static class RightPlan {

        /**
         * 门编号(锁ID)
         */
        private Integer doorNo;

        /**
         * 计划模板编号
         * 同个门不同计划模板采用权限或的方式处理
         * <p>
         * 默认计划模板编号: 65535-7*24小时生效 | 65534-周一到周五24小时生效 | 65533-周六周日24小时生效
         */
        private String planTemplateNo = "1";

        public RightPlan(Integer doorNo) {
            this.doorNo = doorNo;
        }
    }

}
